@@ -1,10 +1,20 @@
 module Aws.Lambda.Runtime where
 
-import Control.Exception (IOException, try)
-import Control.Monad.Except (catchError, throwError)
+import Control.Exception (Exception, IOException, try)
+import Control.Monad.Except (ExceptT, catchError, throwError)
 import Data.Aeson
-import Relude hiding (get, identity)
 import System.Exit (ExitCode (..))
+import qualified Data.Text as Text
+import Data.Text (Text)
+import GHC.Generics
+import qualified Data.ByteString.Lazy as LazyByteString
+import qualified Data.ByteString as ByteString
+import Control.Monad.Trans
+import Text.Read (readMaybe)
+import qualified Data.Text.Encoding    as Encoding
+import Control.Monad
+import Data.Function ((&))
+import Data.Maybe (listToMaybe)
 
 import qualified Data.CaseInsensitive as CI
 import Lens.Micro.Platform hiding ((.=))
@@ -13,9 +23,12 @@ import qualified System.Environment as Environment
 import qualified System.Process as Process
 import qualified Data.UUID as UUID
 import qualified Data.UUID.V4 as UUID
-import System.IO (hFlush)
+import System.IO (hFlush, stdout)
 
 
+type LByteString = LazyByteString.ByteString
+type ByteString = ByteString.ByteString
+
 type App a =
   ExceptT RuntimeError IO a
 
@@ -80,36 +93,36 @@ awsLambdaVersion = "2018-06-01"
 
 nextInvocationEndpoint :: Text -> String
 nextInvocationEndpoint endpoint =
-  "http://" <> toString endpoint <> "/"<> awsLambdaVersion <>"/runtime/invocation/next"
+  "http://" <> Text.unpack endpoint <> "/"<> awsLambdaVersion <>"/runtime/invocation/next"
 
 
 responseEndpoint :: Text -> Text -> String
 responseEndpoint lambdaApi requestId =
-  "http://"<> toString lambdaApi <> "/" <> awsLambdaVersion <> "/runtime/invocation/"<> toString requestId <> "/response"
+  "http://"<> Text.unpack lambdaApi <> "/" <> awsLambdaVersion <> "/runtime/invocation/"<> Text.unpack requestId <> "/response"
 
 
 invocationErrorEndpoint :: Text -> Text -> String
 invocationErrorEndpoint lambdaApi requestId =
-  "http://"<> toString lambdaApi <> "/" <> awsLambdaVersion <> "/runtime/invocation/"<> toString requestId <> "/error"
+  "http://"<> Text.unpack lambdaApi <> "/" <> awsLambdaVersion <> "/runtime/invocation/"<> Text.unpack requestId <> "/error"
 
 
 runtimeInitErrorEndpoint :: Text -> String
 runtimeInitErrorEndpoint lambdaApi =
-  "http://"<> toString lambdaApi <> "/" <> awsLambdaVersion <> "/runtime/init/error"
+  "http://"<> Text.unpack lambdaApi <> "/" <> awsLambdaVersion <> "/runtime/init/error"
 
 
 readEnvironmentVariable :: Text -> App Text
 readEnvironmentVariable envVar = do
-  v <- lift (Environment.lookupEnv $ toString envVar)
+  v <- lift (Environment.lookupEnv $ Text.unpack envVar)
   case v of
     Nothing    -> throwError (EnvironmentVariableNotSet envVar)
-    Just value -> pure (toText value)
+    Just value -> pure (Text.pack value)
 
 
 readFunctionMemory :: App Int
 readFunctionMemory = do
   let envVar = "AWS_LAMBDA_FUNCTION_MEMORY_SIZE"
-  let parseMemory txt = readMaybe (toString txt)
+  let parseMemory txt = readMaybe (Text.unpack txt)
   memoryValue <- readEnvironmentVariable envVar
   case parseMemory memoryValue of
     Just value -> pure value
@@ -130,25 +143,25 @@ getApiData endpoint =
 
 extractHeader :: Wreq.Response LByteString -> Text -> Text
 extractHeader apiData header =
-  decodeUtf8 (apiData ^. Wreq.responseHeader (CI.mk $ encodeUtf8 header))
+  Encoding.decodeUtf8 (apiData ^. Wreq.responseHeader (CI.mk $ Encoding.encodeUtf8 header))
 
 
 extractIntHeader :: Wreq.Response LByteString -> Text -> App Int
 extractIntHeader apiData headerName = do
   let header = extractHeader apiData headerName
-  case readMaybe $ toString header of
+  case readMaybe $ Text.unpack header of
     Nothing    -> throwError (ParseError "deadline" header)
     Just value -> pure value
 
 
 extractBody :: Wreq.Response LByteString -> Text
 extractBody apiData =
-  decodeUtf8 (apiData ^. Wreq.responseBody)
+  Encoding.decodeUtf8 $ LazyByteString.toStrict (apiData ^. Wreq.responseBody)
 
 
 propagateXRayTrace :: Text -> App ()
 propagateXRayTrace xrayTraceId =
-  liftIO $ Environment.setEnv "_X_AMZN_TRACE_ID" $ toString xrayTraceId
+  liftIO $ Environment.setEnv "_X_AMZN_TRACE_ID" $ Text.unpack xrayTraceId
 
 
 initializeContext :: Wreq.Response LByteString -> App Context
@@ -178,58 +191,55 @@ initializeContext apiData = do
 
 getFunctionResult :: UUID.UUID -> Text -> App (Maybe Text)
 getFunctionResult u stdOut = do
-  let out = stdOut
-          & toText
-          & lines
+  let out = Text.lines stdOut
 
   out
    & takeWhile (/= uuid)
-   & traverse_ ( \t -> do
-    liftIO $ putTextLn t
+   & mapM_ ( \t -> do
+    liftIO $ putStrLn $ Text.unpack t
     liftIO $ hFlush stdout)
 
   out
    & dropWhile (/= uuid)
    & dropWhile (== uuid)
-   & nonEmpty
-   & fmap head
+   & listToMaybe
    & return
  where
-  uuid = toText $ UUID.toString u
+  uuid = Text.pack $ UUID.toString u
 
 
 invoke :: Text -> Context -> App LambdaResult
 invoke event context = do
   handlerName <- readEnvironmentVariable "_HANDLER"
   runningDirectory <- readEnvironmentVariable "LAMBDA_TASK_ROOT"
-  let contextJSON = decodeUtf8 $ encode context
+  let contextJSON = Encoding.decodeUtf8 $ LazyByteString.toStrict $ encode context
   uuid <- liftIO UUID.nextRandom
-  out <- liftIO $ Process.readProcessWithExitCode (toString runningDirectory <> "/haskell_lambda")
-                [ "--eventObject", toString event
-                , "--contextObject", contextJSON
-                , "--functionHandler", toString handlerName
+  out <- liftIO $ Process.readProcessWithExitCode (Text.unpack runningDirectory <> "/haskell_lambda")
+                [ "--eventObject", Text.unpack event
+                , "--contextObject", Text.unpack contextJSON
+                , "--functionHandler", Text.unpack handlerName
                 , "--executionUuid", UUID.toString uuid
                 ]
                 ""
   case out of
     (ExitSuccess, stdOut, _) -> do
-      res <- getFunctionResult uuid (toText stdOut)
+      res <- getFunctionResult uuid (Text.pack stdOut)
       case res of
-        Nothing -> throwError (ParseError "parsing result" $ toText stdOut)
+        Nothing -> throwError (ParseError "parsing result" $ Text.pack stdOut)
         Just value -> pure (LambdaResult value)
     (_, stdOut, stdErr)           ->
       if stdErr /= ""
-        then throwError (InvocationError $ toText stdErr)
+        then throwError (InvocationError $ Text.pack stdErr)
         else do
-          res <- getFunctionResult uuid (toText stdOut)
+          res <- getFunctionResult uuid (Text.pack stdOut)
           case res of
-            Nothing -> throwError (ParseError "parsing error" $ toText stdOut)
+            Nothing -> throwError (ParseError "parsing error" $ Text.pack stdOut)
             Just value -> throwError (InvocationError value)
 
 
 publishResult :: Context -> Text -> LambdaResult -> App ()
 publishResult Context {..} lambdaApi (LambdaResult result) =
-  void $ liftIO $ Wreq.post (responseEndpoint lambdaApi awsRequestId) (encodeUtf8 @Text @ByteString result)
+  void $ liftIO $ Wreq.post (responseEndpoint lambdaApi awsRequestId) (Encoding.encodeUtf8 result)
 
 
 invokeAndPublish :: Context -> Text -> Text -> App ()
@@ -240,7 +250,7 @@ invokeAndPublish ctx event lambdaApiEndpoint = do
 
 publishError :: Context -> Text -> RuntimeError -> App ()
 publishError Context {..} lambdaApiEndpoint (InvocationError err) =
-  void (liftIO $ Wreq.post (invocationErrorEndpoint lambdaApiEndpoint awsRequestId) (encodeUtf8 @Text @ByteString err))
+  void (liftIO $ Wreq.post (invocationErrorEndpoint lambdaApiEndpoint awsRequestId) (Encoding.encodeUtf8 err))
 
 publishError Context {..} lambdaApiEndpoint (ParseError t t2) =
   void (liftIO $ Wreq.post (invocationErrorEndpoint lambdaApiEndpoint awsRequestId) (toJSON $ ParseError t t2))
